#include <MD_Output.h>

using namespace ZSET;  

MD_OUTPUT::MD_OUTPUT(STRING n) 
{ 
  name = n; 
  if (!(name == MD_PRINT_TIME || name == MD_PRINT_INCR)) 
    ERROR("Unknown md_output options <"+name+">"); 
  previous_dt = current_dt = 0;
  counter = 0;
  file_name = OUTPUT_FILE_NAME;
  file_cell_name = OUTPUT_CELL_FILE_NAME;
  file_extension = OUTPUT_FILE_EXTENSION;
  restart_name = OUTPUT_RESTART_FILE;
}
MD_OUTPUT::MD_OUTPUT(STRING n, int v, STRING t) {
  name = n;
  it = v;
  print_option = t; 
  //if (!(name == MD_PRINT_TIME || name == MD_PRINT_INCR)) 
  if (!(print_option == PRINT_OPTION_PARTICLE_COORDS_FORCE_VELOCITY ||
        print_option == PRINT_OPTION_PARTICLE_COORDS_FORCE ||
        print_option == PRINT_OPTION_PARTICLE_COORDS_VELOCITY ||
        print_option == PRINT_OPTION_PARTICLE_COORDS  ||
	print_option == PRINT_OPTION_ENERGY))
    ERROR("Unknown **md_output options <"+print_option+">"); 
  previous_dt = current_dt = 0;
  counter = 0;
  file_name = OUTPUT_FILE_NAME;
  file_cell_name = OUTPUT_CELL_FILE_NAME;
  file_extension = OUTPUT_FILE_EXTENSION;
  restart_name = OUTPUT_RESTART_FILE;
}
MD_OUTPUT::MD_OUTPUT(STRING n, double v, STRING t)
{
  name = n;
  dt = v;
  print_option = t; 
  if (!(name == MD_PRINT_TIME || name == MD_PRINT_INCR)) 
    ERROR("Unknown md_output options <"+name+">"); 
  previous_dt = current_dt = 0;
  counter = 0;
  file_name = OUTPUT_FILE_NAME;
  file_cell_name = OUTPUT_CELL_FILE_NAME;
  file_extension = OUTPUT_FILE_EXTENSION;
  restart_name = OUTPUT_RESTART_FILE;
}
void MD_OUTPUT::scale(double ref_time) 
{ 
  if (MD_PRINT_TIME) 
    dt /= ref_time; 
}
void MD_OUTPUT::set_int_time(int t) 
{ 
  it = t; 
}
void MD_OUTPUT::set_print_option(STRING s)
{
  print_option = s;
}
void MD_OUTPUT::set_double_time(double t) 
{ 
  dt = t;
  dt0 = dt; 
}
bool MD_OUTPUT::if_print(int q)  {

  if (name == MD_PRINT_INCR) {
    if (q % it == 0) {
      counter++; 
      return TRUE;
    } 
    else 
      return FALSE;
  }
  else 
   ERROR("Wrong call of print function");

  return FALSE; 
}
bool MD_OUTPUT::if_print(double q)  { 
  if (name == MD_PRINT_TIME)  {
    previous_dt = current_dt; 
    current_dt = q; 
    if (dt >= previous_dt && dt < current_dt) {
      counter++;
      dt += dt0; 
      return TRUE;
    } 
    else 
      return FALSE;
  }
  else ERROR("Wrong call of print function");
  return FALSE;
}
bool MD_OUTPUT::if_print(double dq, int iq)
{
  if (name == MD_PRINT_TIME)
    return if_print(dq);
  else if (name == MD_PRINT_INCR)
    return if_print(iq);
}
void MD_OUTPUT::print(Zofstream& f, const ARRAY<CELL*>& cells, double time, int inc) {
    if (print_option == PRINT_OPTION_ENERGY)
      return;
       f << "# time " << time << " inc/every: "<< inc << "/" << it <<  endl;

       for (int ic = 0; ic < cells.size(); ic++) {
         for (int ip = 0; ip < cells[ic]->particles.size(); ip++) {
           PARTICLE& p = *(cells[ic]->particles[ip]);
           f << p.color << " " << p.x[0] << " " << p.x[1] << " " << p.v[0] << " " << p.v[1] << endl;
	 }
     }
     f << endl;
}



void MD_OUTPUT::print(Zofstream& f, const ARRAY<CELL*>& cells)
{
  if (print_option == PRINT_OPTION_ENERGY)
    return;
//////////////////////////////////////////////////
  if (print_option == PRINT_OPTION_ENERGY)
    return;

  if (print_option==PRINT_OPTION_PARTICLE_COORDS)
  {
     if (cells[0]->size.size() == 2)
     {
       f << "# col X Y" << endl;
       for (int ic = 0; ic < cells.size(); ic++)
         for (int ip = 0; ip < cells[ic]->particles.size(); ip++)
	 {
           PARTICLE& p = *(cells[ic]->particles[ip]);
           f << p.color << " " << p.x[0] << " " << p.x[1] << endl;
	 }
     }
     else 
     {
       f << "# col X Y Z" << endl;
       for (int ic = 0; ic < cells.size(); ic++)
         for (int ip = 0; ip < cells[ic]->particles.size(); ip++)
         {
           PARTICLE& p = *(cells[ic]->particles[ip]);
           f << p.color << " " << p.x[0] << " " << p.x[1] << " " << p.x[2] << endl;
         }
     }
  }
  else if (print_option==PRINT_OPTION_PARTICLE_COORDS_VELOCITY)
  {
     if (cells[0]->size.size() == 2)
     {
       f << "# col X Y Vx Vy" << endl;
       for (int ic = 0; ic < cells.size(); ic++)
         for (int ip = 0; ip < cells[ic]->particles.size(); ip++)
	 {
           PARTICLE& p = *(cells[ic]->particles[ip]);
           f << p.color << " " << p.x[0] << " " << p.x[1] << " " << p.v[0] << " " << p.v[1] << endl;
	 }
     }
     else 
     {
       f << "# col X Y Z" << endl;
       for (int ic = 0; ic < cells.size(); ic++)
         for (int ip = 0; ip < cells[ic]->particles.size(); ip++)
         {
           PARTICLE& p = *(cells[ic]->particles[ip]);
           f << p.color << " " << p.x[0] << " " << p.x[1] << " " << p.x[2] << " " << p.v[0] << " " << p.v[1] << " " << p.v[2] << endl;
         }
     }
  }

/*
  else if (print_option==PRINT_OPTION_PARTICLE_COORDS_FORCE)
  {
     if (cells[0]->size.size() == 2)
     {
       out[i] << "# X Y Fx Fy " << endl;
       for (int ic = 0; ic < cells.size(); ic++)
       {
         for (int ip = 0; ip < cells[ic]->particles.size(); ip++)
	 {
           out[cells[ic]->particles[ip]->color-1] << cells[ic]->particles[ip]->x[0] << " " << cells[ic]->particles[ip]->x[1]  << " " 
						  << cells[ic]->particles[ip]->F[0] << " " << cells[ic]->particles[ip]->F[1] << endl;
	 }
       }
     }
     else 
     {
       out[i] << "# X Y Z Fx Fy Fz" << endl;
       for (int ic = 0; ic < cells.size(); ic++)
         for (int ip = 0; ip < cells[ic]->particles.size(); ip++)
         {
           out[cells[ic]->particles[ip]->color-1] << cells[ic]->particles[ip]->x[0] << " " << cells[ic]->particles[ip]->x[1] << " "
	       << cells[ic]->particles[ip]->x[2] << " " 
	       << cells[ic]->particles[ip]->F[0] << " " << cells[ic]->particles[ip]->F[1] << " " << cells[ic]->particles[ip]->F[2] << endl;
	 }
     }
  }
  else if (print_option==PRINT_OPTION_PARTICLE_COORDS_VELOCITY)
  {
     if (cells[0]->size.size() == 2)
     {
       out[i] << "# X Y Vx Vy" << endl;
       for (int ic = 0; ic < cells.size(); ic++)
       {
         for (int ip = 0; ip < cells[ic]->particles.size(); ip++)
	 {
           out[cells[ic]->particles[ip]->color-1] << cells[ic]->particles[ip]->x[0] << " " << cells[ic]->particles[ip]->x[1]  << " " 
						  << cells[ic]->particles[ip]->v[0] << " " << cells[ic]->particles[ip]->v[1] << endl;
	 }
       }
     }
     else 
     {
       out[i] << "# X Y Z Vx Vy Vz" << endl;
       for (int ic = 0; ic < cells.size(); ic++)
         for (int ip = 0; ip < cells[ic]->particles.size(); ip++)
         {
           out[cells[ic]->particles[ip]->color-1] << cells[ic]->particles[ip]->x[0] << " " << cells[ic]->particles[ip]->x[1] << " "
	       << cells[ic]->particles[ip]->x[2] << " " 
	       << cells[ic]->particles[ip]->v[0] << " " << cells[ic]->particles[ip]->v[1] << " " << cells[ic]->particles[ip]->v[2] << endl;
	 }
     }
  }
  else if (print_option==PRINT_OPTION_PARTICLE_COORDS_FORCE_VELOCITY)
  {
     if (cells[0]->size.size() == 2)
     {
       out[i] << "# X Y Vx Vy Fx Fy" << endl;
       for (int ic = 0; ic < cells.size(); ic++)
       {
         for (int ip = 0; ip < cells[ic]->particles.size(); ip++)
	 {
           out[cells[ic]->particles[ip]->color-1] << cells[ic]->particles[ip]->x[0] << " " << cells[ic]->particles[ip]->x[1] << " " 
						  << cells[ic]->particles[ip]->F[0] << " " << cells[ic]->particles[ip]->F[1] << " "
						  << cells[ic]->particles[ip]->v[0] << " " << cells[ic]->particles[ip]->v[1] << endl;
	 }
       }
     }
     else 
     {
       out[i] << "# X Y Z Vx Vy Vz Fx Fy Fz" << endl;
       for (int ic = 0; ic < cells.size(); ic++)
         for (int ip = 0; ip < cells[ic]->particles.size(); ip++)
         {
           out[cells[ic]->particles[ip]->color-1] << cells[ic]->particles[ip]->x[0] << " " << cells[ic]->particles[ip]->x[1] << " "
	       << cells[ic]->particles[ip]->x[2] << " " 
	       << cells[ic]->particles[ip]->F[0] << " " << cells[ic]->particles[ip]->F[1] << " " << cells[ic]->particles[ip]->F[2] << " "
	       << cells[ic]->particles[ip]->v[0] << " " << cells[ic]->particles[ip]->v[1] << " " << cells[ic]->particles[ip]->v[2] << endl;
	 }
     }
  }
*/
  else
    ERROR("Unknown print option");

//////////////////////////////////////////////////
}
void MD_OUTPUT::print(const ARRAY<CELL*>& cells)
{
    if (counter <= 9)
      file.open((PRINT_OPTION_FOLDER+file_name+"_00000"+itoa(counter)+"."+file_extension)(),ios::out);
    else if (counter <= 99)
      file.open((PRINT_OPTION_FOLDER+file_name+"_0000"+itoa(counter)+"."+file_extension)(),ios::out);
    else if (counter <= 999)
      file.open((PRINT_OPTION_FOLDER+file_name+"_000"+itoa(counter)+"."+file_extension)(),ios::out);
    else if (counter <= 9999)
      file.open((PRINT_OPTION_FOLDER+file_name+"_00"+itoa(counter)+"."+file_extension)(),ios::out);
    else if (counter <= 99999)
      file.open((PRINT_OPTION_FOLDER+file_name+"_0"+itoa(counter)+"."+file_extension)(),ios::out);
    else
      file.open((PRINT_OPTION_FOLDER+file_name+"_"+itoa(counter)+"."+file_extension)(),ios::out);

  print(file,cells);
  
  file.close();
}
void MD_OUTPUT::print_by_cells(const ARRAY<CELL*>& cells, int Nx)
{
    Zofstream out;
    if (counter <= 9)
      out.open((PRINT_OPTION_FOLDER+file_cell_name+"_00000"+itoa(counter)+"."+file_extension)(),ios::out);
    else if (counter <= 99)
      out.open((PRINT_OPTION_FOLDER+file_cell_name+"_0000"+itoa(counter)+"."+file_extension)(),ios::out);
    else if (counter <= 999)
      out.open((PRINT_OPTION_FOLDER+file_cell_name+"_000"+itoa(counter)+"."+file_extension)(),ios::out);
    else if (counter <= 9999)
      out.open((PRINT_OPTION_FOLDER+file_cell_name+"_00"+itoa(counter)+"."+file_extension)(),ios::out);
    else if (counter <= 99999)
      out.open((PRINT_OPTION_FOLDER+file_cell_name+"_0"+itoa(counter)+"."+file_extension)(),ios::out);
    else
      out.open((PRINT_OPTION_FOLDER+file_cell_name+"_"+itoa(counter)+"."+file_extension)(),ios::out);
    out << "# X Y (Z) Vx Vy (Vz) Fx Fy (Fz)" << endl;
    out << "0. 0. 0. 0. 0. 0. 0. 0. 0." << endl;
 
    VECTOR v,f;
    v.resize(cells[0]->size.size());
    f.resize(v.size());
    double m;
    for (int ic = 0; ic < cells.size(); ic++)
    {
      v = 0;
      f = 0;
      m = 0.;
      for (int ip = 0; ip < cells[ic]->particles.size(); ip++) 
      {
        v += cells[ic]->particles[ip]->v;
        f += cells[ic]->particles[ip]->F;
        m += cells[ic]->particles[ip]->mat->mass;
      }
      if (v.size() == 2)
        out << cells[ic]->r0[0] + cells[ic]->size[0]/2. << " " << cells[ic]->r0[1] + cells[ic]->size[1]/2. << " " << m << " " << (v[0]) << " " << (v[1]) 
	    << " " << (f[0]) << " " << (f[1]) << endl;
      else
        out << cells[ic]->r0[0] + cells[ic]->size[0]/2. << " " << cells[ic]->r0[1] + cells[ic]->size[1]/2. << " " << cells[ic]->r0[2] + cells[ic]->size[2]/2. << " " << m << " "
            << v[0] << " " << v[1] << " " << v[2] << " " << f[0] << " " << f[1] << " " << f[2] << endl;
      if ((ic+1) % Nx == 0)
	out << endl;
    }
    out.close();
}
void MD_OUTPUT::print_by_color(int num_col, const ARRAY<CELL*>& cells)
{
  if (print_option == PRINT_OPTION_ENERGY)
    return;
  ARRAY<Zofstream> out(num_col);
  for (int i = 0; i < out.size(); i++)
  {
    if (counter <= 9)
      out[i].open((PRINT_OPTION_FOLDER+file_name+"_c"+itoa(i+1)+"_00000"+itoa(counter)+"."+file_extension)(),ios::out);
    else if (counter <= 99)
      out[i].open((PRINT_OPTION_FOLDER+file_name+"_c"+itoa(i+1)+"_0000"+itoa(counter)+"."+file_extension)(),ios::out);
    else if (counter <= 999)
      out[i].open((PRINT_OPTION_FOLDER+file_name+"_c"+itoa(i+1)+"_000"+itoa(counter)+"."+file_extension)(),ios::out);
    else if (counter <= 9999)
      out[i].open((PRINT_OPTION_FOLDER+file_name+"_c"+itoa(i+1)+"_00"+itoa(counter)+"."+file_extension)(),ios::out);
    else if (counter <= 99999)
      out[i].open((PRINT_OPTION_FOLDER+file_name+"_c"+itoa(i+1)+"_0"+itoa(counter)+"."+file_extension)(),ios::out);
    else
      out[i].open((PRINT_OPTION_FOLDER+file_name+"_c"+itoa(i+1)+"_"+itoa(counter)+"."+file_extension)(),ios::out);
  }

  if (print_option==PRINT_OPTION_PARTICLE_COORDS)
  {
     if (cells[0]->size.size() == 2)
     {
//       out[i] << "# X Y" << endl;
       for (int ic = 0; ic < cells.size(); ic++)
       {
         for (int ip = 0; ip < cells[ic]->particles.size(); ip++)
	 {
           out[cells[ic]->particles[ip]->color-1] << cells[ic]->particles[ip]->x[0] << " " << cells[ic]->particles[ip]->x[1] << endl;
	 }
       }
     }
     else 
     {
//       out[i] << "# X Y Z" << endl;
       for (int ic = 0; ic < cells.size(); ic++)
         for (int ip = 0; ip < cells[ic]->particles.size(); ip++)
           out[cells[ic]->particles[ip]->color-1] << cells[ic]->particles[ip]->x[0] << " " << cells[ic]->particles[ip]->x[1] << " "
	       << cells[ic]->particles[ip]->x[2] << endl;
     }
  }
  else if (print_option==PRINT_OPTION_PARTICLE_COORDS_FORCE)
  {
     if (cells[0]->size.size() == 2)
     {
//       out[i] << "# X Y Fx Fy " << endl;
       for (int ic = 0; ic < cells.size(); ic++)
       {
         for (int ip = 0; ip < cells[ic]->particles.size(); ip++)
	 {
           out[cells[ic]->particles[ip]->color-1] << cells[ic]->particles[ip]->x[0] << " " << cells[ic]->particles[ip]->x[1]  << " " 
						  << cells[ic]->particles[ip]->F[0] << " " << cells[ic]->particles[ip]->F[1] << endl;
	 }
       }
     }
     else 
     {
//       out[i] << "# X Y Z Fx Fy Fz" << endl;
       for (int ic = 0; ic < cells.size(); ic++)
         for (int ip = 0; ip < cells[ic]->particles.size(); ip++)
         {
           out[cells[ic]->particles[ip]->color-1] << cells[ic]->particles[ip]->x[0] << " " << cells[ic]->particles[ip]->x[1] << " "
	       << cells[ic]->particles[ip]->x[2] << " " 
	       << cells[ic]->particles[ip]->F[0] << " " << cells[ic]->particles[ip]->F[1] << " " << cells[ic]->particles[ip]->F[2] << endl;
	 }
     }
  }
  else if (print_option==PRINT_OPTION_PARTICLE_COORDS_VELOCITY)
  {
     if (cells[0]->size.size() == 2)
     {
//       out[i] << "# X Y Vx Vy" << endl;
       for (int ic = 0; ic < cells.size(); ic++)
       {
         for (int ip = 0; ip < cells[ic]->particles.size(); ip++)
	 {
           out[cells[ic]->particles[ip]->color-1] << cells[ic]->particles[ip]->x[0] << " " << cells[ic]->particles[ip]->x[1]  << " " 
						  << cells[ic]->particles[ip]->v[0] << " " << cells[ic]->particles[ip]->v[1] << endl;
	 }
       }
     }
     else 
     {
//       out[i] << "# X Y Z Vx Vy Vz" << endl;
       for (int ic = 0; ic < cells.size(); ic++)
         for (int ip = 0; ip < cells[ic]->particles.size(); ip++)
         {
           out[cells[ic]->particles[ip]->color-1] << cells[ic]->particles[ip]->x[0] << " " << cells[ic]->particles[ip]->x[1] << " "
	       << cells[ic]->particles[ip]->x[2] << " " 
	       << cells[ic]->particles[ip]->v[0] << " " << cells[ic]->particles[ip]->v[1] << " " << cells[ic]->particles[ip]->v[2] << endl;
	 }
     }
  }
  else if (print_option==PRINT_OPTION_PARTICLE_COORDS_FORCE_VELOCITY)
  {
     if (cells[0]->size.size() == 2)
     {
//       out[i] << "# X Y Vx Vy Fx Fy" << endl;
       for (int ic = 0; ic < cells.size(); ic++)
       {
         for (int ip = 0; ip < cells[ic]->particles.size(); ip++)
	 {
           out[cells[ic]->particles[ip]->color-1] << cells[ic]->particles[ip]->x[0] << " " << cells[ic]->particles[ip]->x[1] << " " 
						  << cells[ic]->particles[ip]->F[0] << " " << cells[ic]->particles[ip]->F[1] << " "
						  << cells[ic]->particles[ip]->v[0] << " " << cells[ic]->particles[ip]->v[1] << endl;
	 }
       }
     }
     else 
     {
//       out[i] << "# X Y Z Vx Vy Vz Fx Fy Fz" << endl;
       for (int ic = 0; ic < cells.size(); ic++)
         for (int ip = 0; ip < cells[ic]->particles.size(); ip++)
         {
           out[cells[ic]->particles[ip]->color-1] << cells[ic]->particles[ip]->x[0] << " " << cells[ic]->particles[ip]->x[1] << " "
	       << cells[ic]->particles[ip]->x[2] << " " 
	       << cells[ic]->particles[ip]->F[0] << " " << cells[ic]->particles[ip]->F[1] << " " << cells[ic]->particles[ip]->F[2] << " "
	       << cells[ic]->particles[ip]->v[0] << " " << cells[ic]->particles[ip]->v[1] << " " << cells[ic]->particles[ip]->v[2] << endl;
	 }
     }
  }
  else
    ERROR("Unknown print option");
}
void MD_OUTPUT::print(MD_MESH* mesh, const ARRAY<PARTICLE*> particles) {
    if (counter <= 9)
      file.open((PRINT_OPTION_FOLDER+STRING(OUTPUT_MESH_NAME)+"_00000"+itoa(counter)+"."+file_extension)(),ios::out);
    else if (counter <= 99)
      file.open((PRINT_OPTION_FOLDER+STRING(OUTPUT_MESH_NAME)+"_0000"+itoa(counter)+"."+file_extension)(),ios::out);
    else if (counter <= 999)
      file.open((PRINT_OPTION_FOLDER+STRING(OUTPUT_MESH_NAME)+"_000"+itoa(counter)+"."+file_extension)(),ios::out);
    else if (counter <= 9999)
      file.open((PRINT_OPTION_FOLDER+STRING(OUTPUT_MESH_NAME)+"_00"+itoa(counter)+"."+file_extension)(),ios::out);
    else if (counter <= 99999)
      file.open((PRINT_OPTION_FOLDER+STRING(OUTPUT_MESH_NAME)+"_0"+itoa(counter)+"."+file_extension)(),ios::out);
    else
      file.open((PRINT_OPTION_FOLDER+STRING(OUTPUT_MESH_NAME)+"_"+itoa(counter)+"."+file_extension)(),ios::out);

  VECTOR rc = mesh->get_element(0)->get_coord0(0), rv, rb;
  double phase;
  for (int i = 0; i < mesh->size(); i++) {
    MD_ELEMENT& el = *(mesh->get_element(i));
    rv = el.get_coord0(0);
    rb = particles[el.get_particle(4)]->x;
    if (rv != rc)
    {
      VECTOR r = particles[mesh->get_element(i-1)->get_particle(1)]->x;
      file << r[0] << " " << r[1] << " 0" << endl << endl;
    }
    ARRAY<double> dist(4);
    double ref = (el.get_coord0(0)-el.get_coord0(2)).norm();
    for (int j = 0; j < el.size(); j++)
    {
       VECTOR r = particles[el.get_particle(j)]->x;
       dist[j] = (r - rb).norm()/ref;
    }
    if (dist[0] - dist[1] > 0.2 && dist[0] - dist[2] > 0.2 && dist[0] - dist[3] > 0.2) 
      phase = 5; 
    else if (dist[1] - dist[0] > 0.2 && dist[1] - dist[2] > 0.2 && dist[1] - dist[3] > 0.2)
      phase = 6; 
    else if (dist[2] - dist[0] > 0.2 && dist[2] - dist[1] > 0.2 && dist[2] - dist[3] > 0.2)
      phase = 7; 
    else if (dist[3] - dist[0] > 0.2 && dist[3] - dist[1] > 0.2 && dist[3] - dist[2] > 0.2)
      phase = 8; 
    else if (dist[0] - dist[1] > 0.1 && dist[0] - dist[2] > 0.1 && dist[0] - dist[3] > 0.1)
      phase = 1;
    else if (dist[1] - dist[0] > 0.1 && dist[1] - dist[2] > 0.1 && dist[1] - dist[3] > 0.1)
      phase = 2;
    else if (dist[2] - dist[0] > 0.1 && dist[2] - dist[1] > 0.1 && dist[2] - dist[3] > 0.1)
      phase = 3;
    else if (dist[3] - dist[0] > 0.1 && dist[3] - dist[1] > 0.1 && dist[3] - dist[2] > 0.1)
      phase = 4;
    else 
      phase = 0;
    for (int j = 0; j < 1/*el.size()-1*/; j++)
    {
       VECTOR r = particles[el.get_particle(j)]->x;
       file << r[0] << " " << r[1] << " " << phase << endl;
    }
    //file << el.get_coord0(0)[0] << " " << el.get_coord0(0)[1] << " " << phase << endl;/* << endl;
/*
    file << el.get_coord0(1)[0] << " " << el.get_coord0(1)[1] << " " << phase << endl;
    file << el.get_coord0(1)[0]+1e-3 << " " << el.get_coord0(1)[1] << " " << phase << endl;
    file << el.get_coord0(2)[0]+1e-3 << " " << el.get_coord0(2)[1] << " " << phase << endl;
    file << el.get_coord0(2)[0] << " " << el.get_coord0(2)[1] << " " << phase << endl;
    file << el.get_coord0(1)[0] << " " << el.get_coord0(1)[1] << " " << phase << endl;*/
    rc = el.get_coord0(1);
  }

  file.close(); 
/*
    int back = -1,start = 0,mult=1,ci=0,color;
    for (int i = 0; i < mesh.size(); i++)
    {
      if (mesh[i] == -1)
      {
        back *= -1;
	start = i+1;
	mult=2;
        ci=-1;
      }   
      prn("back = ", back);	
      prn("start = ", start);
      if (back < 0)
      {
        if (start+2*Nx < mesh.size() && mesh[i+1] >= 0 && mesh[i] >= 0 && mesh[start+2*Nx-i] >= 0.) 
        {
          VECTOR x0 = particles[mesh[i+1]]->x, x1 = particles[mesh[i]]->x, x2 = particles[mesh[start+2*Nx-ci]]->x;
          double shear = ((x0-x1)|(x2-x1))/(x0-x1).norm()/(x2-x1).norm();
          prn("back < 0: x0 = ", x0);
          prn("        : x1 = ", x1);
          prn("        : x2 = ", x2);
          prn("        : v1 = ", x0-x1);
          prn("        : v2 = ", x2-x1);
          prn("        : pr = ", shear);
          if (shear < -0.1)
	    color = 2;
          else if (shear > 0.1)
            color = 3;
          else
            color = 1;

          file << x0[0] << " " << x0[1] << " " << shear << endl
               << x1[0] << " " << x1[1] << " " << shear << endl
               << x2[0] << " " << x2[1] << " " << shear << endl
	       << x0[0] << " " << x0[1] << " " << shear << endl << endl; 
        }
      }
      else 
      {
        if (start+2*Nx < mesh.size() && mesh[i+1] >= 0 && mesh[i] >= 0 && mesh[start+2*Nx-i] >= 0.)
        {
          VECTOR x0 = particles[mesh[i+1]]->x, x1 = particles[mesh[i]]->x, x2 = particles[mesh[start+2*Nx-ci]]->x;
	  double shear = -((x0-x1)|(x2-x1))/(x0-x1).norm()/(x2-x1).norm();
          prn("back > 0: x0 = ", x0);
          prn("        : x1 = ", x1);
          prn("        : x2 = ", x2);
          prn("        : v1 = ", x0-x1);
          prn("        : v2 = ", x2-x1);
          prn("        : pr = ", shear);
          if (shear > 0.1)
	    color = 2;
          else if (shear < -0.1)
            color = 3;
          else
            color = 1;
          
          file << x0[0] << " " << x0[1] << " " << shear << endl
               << x1[0] << " " << x1[1] << " " << shear << endl
               << x2[0] << " " << x2[1] << " " << shear << endl
	       << x0[0] << " " << x0[1] << " " << shear << endl << endl; 
        }
      }
      ci++;  
    } 
*/
}
void MD_OUTPUT::print_integrals(Zofstream& file, const ARRAY<CELL*>& cells, MD_POTENTIAL* pot, double time)
{
  double kin_energy = 0, pot_energy = 0;
  evaluate_potential_energy(cells, pot, pot_energy);
  for (int i = 0; i < cells.size(); i++) {
    for (int j = 0; j < cells[i]->particles.size(); j++) {
	kin_energy += cells[i]->particles[j]->mat->mass * pow(cells[i]->particles[j]->v.norm(),2.)/2.;
    }
  }
//  if (kin_energy > 1e10)
//    ERROR("Kinetic energy is too high!");
  file << time << " " << kin_energy << " " << pot_energy << endl;
}
void MD_OUTPUT::print_integrals(Zofstream& file, double kinetic_energy, double potential_energy, double pre_pressure, double time, int num_particles, double mult, int dim) {
   double Temp = 2./(dim*num_particles)*kinetic_energy*(mult/kB);
   file << time << " " << kinetic_energy << " " << -1*potential_energy << " " << Temp << " " <<  pre_pressure + 2*kinetic_energy << endl; 
}
void MD_OUTPUT::evaluate_potential_energy(const ARRAY<CELL*>& cells, MD_POTENTIAL* pot, double& pot_energy) {
    if (pot->get_name() == LJ) {
      VECTOR F;
      ARRAY<int> neighbours;
      for (int icell = 0; icell < cells.size(); icell++) {
	neighbours = cells[icell]->get_neighbours();
        cells[icell]->checked = TRUE;
        for (int ip = 0; ip < cells[icell]->particles.size(); ip++)
        {
	  PARTICLE& pi = *(cells[icell]->particles[ip]);
// Check in the current cell
          for (int jp = ip+1; jp < cells[icell]->particles.size(); jp++)
          {
	     PARTICLE& pj = *(cells[icell]->particles[jp]);
             pot_energy += pot->get_potential(&pi,&pj);
	  }
// Check in neighbouring cells
	  for (int in = 0; in < neighbours.size(); in++)
          {
	    if (!cells[neighbours[in]]->checked)
            {
		for (int jp = 0; jp < cells[neighbours[in]]->particles.size(); jp++)
                {
             	  PARTICLE& pj = *(cells[neighbours[in]]->particles[jp]);
             	  pot_energy += pot->get_potential(&pi,&pj);
          	}
	    }
          }
        }
      }
      for (int icell = 0; icell < cells.size(); icell++)
        cells[icell]->checked = FALSE;
    }
    else
	ERROR("Implemented only for LJ potential");  
}
void MD_OUTPUT::write_restart(const ARRAY<PARTICLE*> p) {
  Zofstream f;
  f.open((PRINT_OPTION_FOLDER+restart_name)(),ios::out);
  int dim = 0;
  if (p[0] != NULL)
    dim = p[0]->x.size();
  else
    ERROR("Restart file cannot be written. There is no particles.");
  f << "# Restart file format:" << endl; 
  f << "#   ***particles problem_dim (int)" << endl;
  f << "#        number_of_particles (int)" << endl;
  f << "# Data format: id x y (z) vx vy (vz) color bc_u bc_v bc_f" << endl;
  f << "***particles\n" << dim << " " << p.size() << endl;
  for (int i = 0; i < p.size(); i++) {
    f << p[i]->id;
    for (int j = 0; j < dim; j++) {
      if (p[i]->x[j] == 0)
	f << " 0.0";
     else
        f << " " << dtoa(p[i]->x[j],"%10.25g");
    }
    for (int j = 0; j < dim; j++) {
      if (p[i]->v[j] == 0)
	f << " 0.0";
      else
        f << " " << dtoa(p[i]->v[j],"%10.25g");
    }
    f << " " << p[i]->color;
    if (p[i]->bc_x.size() > 0)
      f << " " << dim;
    else 
      f << " " << 0;
    if (p[i]->bc_v.size() > 0)
      f << " " << dim;
    else 
      f << " " << 0;
    if (p[i]->bc_f.size() > 0)
      f << " " << dim << endl;
    else 
      f << " " << 0 << endl;
  }
  f.close();
}

