#ifndef __MD_OUTPUT__
#define __MD_OUTPUT__

#include <Consts.h>
#include <Stringpp.h>
#include <Cell.h>
#include <Print.h>
#include <MD_Potential.h>
#include <MD_Mesh.h>

namespace ZSET {
  using namespace ZSET;

class MD_OUTPUT
{
protected:
  STRING name;
  STRING print_option, file_name, file_cell_name, file_extension, restart_name;
  int    it, current_it, counter;
  double dt, dt0, previous_dt, current_dt;
  Zofstream file;
public:

  MD_OUTPUT(STRING n);
  MD_OUTPUT(STRING n, int v, STRING t);
  MD_OUTPUT(STRING n, double v, STRING t);
  MD_OUTPUT() {}
 ~MD_OUTPUT() {}

  STRING get_type() { return name; }  
  void scale(double ref_time);
  void set_print_option(STRING s);
  void set_int_time(int t);
  void set_double_time(double t);
  bool if_print(double dq, int iq);
  bool if_print(int q);
  bool if_print(double q);
  void print(Zofstream& f, const ARRAY<CELL*>& cells, double t, int inc);
  void print(Zofstream& f, const ARRAY<CELL*>& cells);
  void write_restart(const ARRAY<PARTICLE*> p);
  void print(const ARRAY<CELL*>& cells);
  void print(MD_MESH* mesh, const ARRAY<PARTICLE*> p);
  void print_by_color(int num_col, const ARRAY<CELL*>& cells);
  void print_by_cells(const ARRAY<CELL*>& cells, int Nx);
  void print_integrals(Zofstream& file, const ARRAY<CELL*>& cells, MD_POTENTIAL* pot, double time);
  void print_integrals(Zofstream& file, double kinetic_energy, double potential_energy, double pre_pressure, double time, int num_particles, double mult, int dim); 
  void evaluate_potential_energy(const ARRAY<CELL*>& cells, MD_POTENTIAL* pot, double& pot_energy);
};

}

#endif
